@@ -1,16 +1,19 @@
 package Test::WWW::Mechanize;
 
+use strict;
+use warnings;
+
 =head1 NAME
 
 Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.16
+Version 1.18
 
 =cut
 
-our $VERSION = '1.16';
+our $VERSION = '1.20';
 
 =head1 SYNOPSIS
 
@@ -42,10 +45,24 @@ This is equivalent to:
 
 but has nicer diagnostics if they fail.
 
-=cut
+Default descriptions will be supplied for most methods if you omit them. e.g.
 
-use warnings;
-use strict;
+    my $mech = Test::WWW::Mechanize->new;
+    $mech->get_ok( 'http://petdance.com/' );
+    $mech->base_is( 'http://petdance.com/' );
+    $mech->title_is( "Invoice Status" );
+    $mech->content_contains( "Andy Lester" );
+    $mech->content_like( qr/(cpan|perl)\.org/ );
+
+results in
+
+    ok - Got 'http://petdance.com/' ok
+    ok - Base is 'http://petdance.com/'
+    ok - Title is 'Invoice Status'
+    ok - Content contains 'Andy Lester'
+    ok - Content is like '(?-xism:(cpan|perl)\.org)'
+
+=cut
 
 use WWW::Mechanize ();
 use Test::LongString;
@@ -103,11 +120,11 @@ sub get_ok {
             $desc = shift;
         }
         elsif ( ref $flex eq 'HASH' ) {
-            %opts = %$flex;
+            %opts = %{$flex};
             $desc = shift;
         }
         elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @$flex;
+            %opts = @{$flex};
             $desc = shift;
         }
         else {
@@ -118,7 +135,10 @@ sub get_ok {
     $self->get( $url, %opts );
     my $ok = $self->success;
 
-    $desc = "GET $url" unless defined $desc;
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "GET $url";
+    }
     $Test->ok( $ok, $desc );
     if ( !$ok ) {
         $Test->diag( $self->status );
@@ -153,11 +173,11 @@ sub post_ok {
             $desc = shift;
         }
         elsif ( ref $flex eq 'HASH' ) {
-            %opts = %$flex;
+            %opts = %{$flex};
             $desc = shift;
         }
         elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @$flex;
+            %opts = @{$flex};
             $desc = shift;
         }
         else {
@@ -165,7 +185,10 @@ sub post_ok {
         }
     } # parms left
 
-    $desc = "POST to $url" unless defined $desc;
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "POST $url";
+    }
     $self->post( $url, \%opts );
     my $ok = $self->success;
     $Test->ok( $ok, $desc );
@@ -178,7 +201,7 @@ sub post_ok {
 }
 
 
-=head2 submit_form_ok( \%parms [, $comment] )
+=head2 submit_form_ok( \%parms [, $desc] )
 
 Makes a C<submit_form()> call and executes tests on the results.
 The form must be found, and then submitted successfully.  Otherwise,
@@ -190,7 +213,7 @@ this function are a hashref.  You have to call this function like:
 
     $agent->submit_form_ok( {n=>3}, "looking for 3rd link" );
 
-As with other test functions, C<$comment> is optional.  If it is supplied
+As with other test functions, C<$desc> is optional.  If it is supplied
 then it will display when running the test harness in verbose mode.
 
 Returns true value if the specified link was found and followed
@@ -202,14 +225,14 @@ is not available.
 sub submit_form_ok {
     my $self = shift;
     my $parms = shift || {};
-    my $comment = shift;
+    my $desc = shift;
 
     if ( ref $parms ne 'HASH' ) {
        Carp::croak "FATAL: parameters must be given as a hashref";
     }
 
     # return from submit_form() is an HTTP::Response or undef
-    my $response = $self->submit_form( %$parms );
+    my $response = $self->submit_form( %{$parms} );
 
     my $ok;
     my $error;
@@ -225,14 +248,14 @@ sub submit_form_ok {
         }
     }
 
-    $Test->ok( $ok, $comment );
+    $Test->ok( $ok, $desc );
     $Test->diag( $error ) if $error;
 
     return $ok;
 }
 
 
-=head2 $mech->follow_link_ok( \%parms [, $comment] )
+=head2 $mech->follow_link_ok( \%parms [, $desc] )
 
 Makes a C<follow_link()> call and executes tests on the results.
 The link must be found, and then followed successfully.  Otherwise,
@@ -244,7 +267,7 @@ this function are a hashref.  You have to call this function like:
 
     $mech->follow_link_ok( {n=>3}, "looking for 3rd link" );
 
-As with other test functions, C<$comment> is optional.  If it is supplied
+As with other test functions, C<$desc> is optional.  If it is supplied
 then it will display when running the test harness in verbose mode.
 
 Returns a true value if the specified link was found and followed
@@ -256,14 +279,19 @@ is not available.
 sub follow_link_ok {
     my $self = shift;
     my $parms = shift || {};
-    my $comment = shift;
+    my $desc = shift;
+
+    if (!defined($desc)) {
+        my $parms_str = join(", ", map { join("=", $_, $parms->{$_}) } keys(%{$parms}));
+        $desc = "Followed link with '$parms_str'" if !defined($desc);
+    }
 
     if ( ref $parms ne 'HASH' ) {
        Carp::croak "FATAL: parameters must be given as a hashref";
     }
 
     # return from follow_link() is an HTTP::Response or undef
-    my $response = $self->follow_link( %$parms );
+    my $response = $self->follow_link( %{$parms} );
 
     my $ok;
     my $error;
@@ -279,7 +307,7 @@ sub follow_link_ok {
         }
     }
 
-    $Test->ok( $ok, $comment );
+    $Test->ok( $ok, $desc );
     $Test->diag( $error ) if $error;
 
     return $ok;
@@ -287,12 +315,13 @@ sub follow_link_ok {
 
 =head1 METHODS: CONTENT CHECKING
 
-=head2 html_lint_ok( [$msg] )
+=head2 $mech->html_lint_ok( [$msg] )
 
 Checks the validity of the HTML on the current page.  If the page is not
-HTML, then it fails.
+HTML, then it fails.  The URI is automatically appended to the I<$msg>.
 
-The URI is automatically appended to the I<$msg>.
+Note that HTML::Lint must be installed for this to work.  Otherwise,
+it will blow up.
 
 =cut
 
@@ -300,22 +329,27 @@ sub html_lint_ok {
     my $self = shift;
     my $msg = shift;
 
+    eval 'require HTML::Lint';
+    $@ and die 'html_lint_ok cannot run without HTML::Lint';
+
     my $uri = $self->uri;
     $msg = $msg ? "$msg ($uri)" : $uri;
 
     my $ok;
 
     if ( $self->is_html ) {
-        require HTML::Lint;
 
         my $lint = HTML::Lint->new;
-        $lint->newfile( $uri );
         $lint->parse( $self->content );
 
         my @errors = $lint->errors;
-        if ( @errors ) {
+        my $nerrors = @errors;
+        if ( $nerrors ) {
             $ok = $Test->ok( 0, $msg );
+            $Test->diag( "HTML::Lint errors for $uri" );
             $Test->diag( $_->as_string ) for @errors;
+            my $s = $nerrors == 1 ? '' : 's';
+            $Test->diag( "$nerrors error$s on the page" );
         }
         else {
             $ok = $Test->ok( 1, $msg );
@@ -342,6 +376,7 @@ sub title_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
+    $desc = "Title is '$str'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->title, $str, $desc );
@@ -359,6 +394,7 @@ sub title_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Title is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->title, $regex, $desc );
@@ -376,6 +412,7 @@ sub title_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Title unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->title, $regex, $desc );
@@ -393,6 +430,7 @@ sub base_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
+    $desc = "Base is '$str'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->base, $str, $desc );
@@ -410,6 +448,7 @@ sub base_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Base is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->base, $regex, $desc );
@@ -427,6 +466,7 @@ sub base_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Base unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->base, $regex, $desc );
@@ -444,6 +484,8 @@ sub content_is {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    $desc = qq{Content is "$str"} if !defined($desc);
+
     return is_string( $self->content, $str, $desc );
 }
 
@@ -459,6 +501,11 @@ sub content_contains {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) eq 'REGEX' ) {
+        diag( "content_contains takes a string, not a regex" );
+    }
+    $desc = qq{Content contains "$str"} if !defined($desc);
+
     return contains_string( $self->content, $str, $desc );
 }
 
@@ -474,6 +521,11 @@ sub content_lacks {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) eq 'REGEX' ) {
+        diag( 'content_lacks takes a string, not a regex' );
+    }
+    $desc = qq{Content lacks "$str"} if !defined($desc);
+
     return lacks_string( $self->content, $str, $desc );
 }
 
@@ -487,6 +539,7 @@ sub content_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Content is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->content, $regex, $desc );
@@ -502,6 +555,7 @@ sub content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Content unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->content, $regex, $desc );
@@ -518,6 +572,7 @@ sub has_tag {
     my $tag  = shift;
     my $text = shift;
     my $desc = shift;
+    $desc = "Page has $tag tag with '$text'" if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $text eq $_[0] } );
 
@@ -536,6 +591,7 @@ sub has_tag_like {
     my $tag  = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Page has $tag tag like '$regex'" if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $_[0] =~ $regex } );
 
@@ -582,6 +638,8 @@ sub page_links_ok {
     my $self = shift;
     my $desc = shift;
 
+    $desc = 'All links ok' unless defined $desc;
+
     my @links = $self->followable_links();
     my @urls = _format_links(\@links);
 
@@ -608,10 +666,12 @@ sub page_links_content_like {
     my $regex = shift;
     my $desc = shift;
 
+    $desc = qq{All links are like "$regex"} unless defined $desc;
+
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
         my $ok = $Test->ok( 0, 'page_links_content_like' );
-        $Test->diag("     '$regex' doesn't look much like a regex to me.");
+        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -641,11 +701,12 @@ sub page_links_content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "All links are unlike '$regex'" if !defined($desc);
 
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
         my $ok = $Test->ok( 0, 'page_links_content_unlike' );
-        $Test->diag("     '$regex' doesn't look much like a regex to me.");
+        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -684,6 +745,7 @@ sub links_ok {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, 'are ok') unless defined $desc;
     my @failures = $self->_check_links_status( \@urls );
     my $ok = (@failures == 0);
 
@@ -713,6 +775,7 @@ sub link_status_is {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "have status $status") if !defined($desc);
     my @failures = $self->_check_links_status( \@urls, $status );
     my $ok = (@failures == 0);
 
@@ -742,6 +805,7 @@ sub link_status_isnt {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "do not have status $status") if !defined($desc);
     my @failures = $self->_check_links_status( \@urls, $status, 'isnt' );
     my $ok = (@failures == 0);
 
@@ -774,11 +838,12 @@ sub link_content_like {
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
         my $ok = $Test->ok( 0, 'link_content_like' );
-        $Test->diag("     '$regex' doesn't look much like a regex to me.");
+        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "are like '$regex'") if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex );
     my $ok = (@failures == 0);
 
@@ -810,11 +875,12 @@ sub link_content_unlike {
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
         my $ok = $Test->ok( 0, 'link_content_unlike' );
-        $Test->diag("     '$regex' doesn't look much like a regex to me.");
+        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, qq{are not like "$regex"}) if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex, 'unlike' );
     my $ok = (@failures == 0);
 
@@ -824,6 +890,13 @@ sub link_content_unlike {
     return $ok;
 }
 
+# Create a default description for the link_* methods, including the link count.
+sub _default_links_desc {
+    my ($urls, $desc_suffix) = @_;
+    my $url_count = scalar(@$urls);
+    return sprintf("%d link%s %s", $url_count, $url_count == 1 ? "" : "s", $desc_suffix);
+}
+
 # This actually performs the status check of each url.
 sub _check_links_status {
     my $self = shift;
@@ -891,13 +964,13 @@ sub _format_links {
     my $links = shift;
 
     my @urls;
-    if(ref($links) eq 'ARRAY') {
-        if(defined($$links[0])) {
-            if(ref($$links[0]) eq 'WWW::Mechanize::Link') {
-                @urls=map { $_->url() } @$links;
+    if (ref($links) eq 'ARRAY') {
+        if (defined($$links[0])) {
+            if (ref($$links[0]) eq 'WWW::Mechanize::Link') {
+                @urls = map { $_->url() } @{$links};
             }
             else {
-                @urls=@$links;
+                @urls = @{$links};
             }
         }
     }
@@ -967,7 +1040,7 @@ sub stuff_inputs {
 
     my $options = shift || {};
     assert_isa( $options, 'HASH' );
-    assert_in( $_, ['ignore', 'fill', 'specs'] ) foreach ( keys %$options );
+    assert_in( $_, ['ignore', 'fill', 'specs'] ) foreach ( keys %{$options} );
 
     # set up the fill we'll use unless a field overrides it
     my $default_fill = '@';
@@ -986,13 +1059,13 @@ sub stuff_inputs {
     if ( exists $options->{specs} ) {
         assert_isa( $options->{specs}, 'HASH' );
         $specs = $options->{specs};
-        foreach my $field_name ( keys %$specs ) {
+        foreach my $field_name ( keys %{$specs} ) {
             assert_isa( $specs->{$field_name}, 'HASH' );
             assert_in( $_, ['fill', 'maxlength'] ) foreach ( keys %{$specs->{$field_name}} );
         }
     }
 
-    my @inputs = $self->find_all_inputs( type => qr/^(text|textarea|password)$/ );
+    my @inputs = $self->find_all_inputs( type_regex => qr/^(text|textarea|password)$/ );
 
     foreach my $field ( @inputs ) {
         next if $field->readonly();
@@ -1075,8 +1148,8 @@ You can also look for information at:
 
 L<http://code.google.com/p/www-mechanize/issues/list>
 
-Please note that WWW::Mechanize and Test::WWW::Mechanize do NOT use
-rt.cpan.org at
+Please B<do not use> the old queues for WWW::Mechanize and
+Test::WWW::Mechanize at
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-WWW-Mechanize>
 
 =item * AnnoCPAN: Annotated CPAN documentation
